import jarvis
from discord import User
from discord.ext import commands
from discord_slash import cog_ext, SlashContext
from jarvis.utils.db import DBManager


class AdminCog(commands.Cog):
    """
    Guild admin functions

    Used to manage guilds
    """

    def __init__(self, bot):
        self.bot = bot
        config = jarvis.config.get_config()
        self.db = DBManager(config.mongo)

    async def _ban(self, ctx: SlashContext, user: User = None, reason=None):
        if not user or user == ctx.message.author:
            await ctx.send("You cannot ban yourself.")
            if not reason:
                reason = (
                    "Mr. Stark is displeased with your presence. Please leave."
                )
                guild_name = ctx.message.guild.name
                await user.send(
                    f"You have been banned from {guild_name}. Reason:\n{reason}"
                )
                await ctx.guild.ban(user, reason=reason)
                await ctx.send(
                    f"{user.name} has been banned from {guild_name}."
                    + f"Reason:\n{reason}"
                )

    @cog_ext.cog_slash(
        name="ban", description="Ban a user", guild_ids=[578757004059738142]
    )
    @commands.has_permissions(ban_members=True)
    async def _ban_slash(self, ctx, user: User = None, reason=None):
        await self._ban(ctx, user, reason)

    @commands.command(name="ban")
    @commands.has_permissions(ban_members=True)
    async def _ban_pref(self, ctx, user: User = None, reason=None):
        await self._ban(ctx, user, reason)

    async def _kick(self, ctx, user: User = None, reason=None):
        if not user or user == ctx.message.author:
            await ctx.send("You cannot kick yourself.")
            if not reason:
                reason = (
                    "Mr. Stark is displeased with your presence. Please leave."
                )
                guild_name = ctx.message.guild.name
                try:
                    await user.send(
                        f"You have been kicked from {guild_name}. Reason:\n{reason}"
                    )
                except Exception:
                    await ctx.send("Unable to message user.")
                await ctx.guild.kick(user, reason=reason)
                await ctx.send(
                    f"{user.name} has been kicked from {guild_name}."
                    + f"Reason:\n{reason}"
                )

    @commands.command(name="kick")
    @commands.has_permissions(kick_members=True)
    async def _kick_pref(self, ctx, user: User = None, reason=None):
        await self._kick(ctx, user, reason)

    @cog_ext.cog_slash(
        name="kick", description="Kick a user", guild_ids=[578757004059738142]
    )
    @commands.has_permissions(kick_members=True)
    async def _kick_slash(self, ctx, user: User = None, reason=None):
        await self._kick(ctx, user, reason)


def setup(bot):
    bot.add_cog(AdminCog(bot))
